/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#define DBUS_API_SUBJECT_TO_CHANGE

#include "maman-file.h"

#include <string.h>
#include <dbus/dbus.h>

struct _MamanFilePrivate
{
	DBusConnection *dbus_connection;	
};

static DBusHandlerResult maman_file_handle_icd_message(DBusConnection *c,
							DBusMessage *msg,
							gpointer user_data);
static DBusConnection *maman_file_get_dbus_connection(MamanFile *file);

static void
maman_file_instance_init (GTypeInstance   *instance,
                         gpointer         g_class)
{
        MamanFile *self = (MamanFile *)instance;
        self->private = g_new (MamanFilePrivate, 1);
}

static void
maman_file_class_init (gpointer g_class,
                      gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
        MamanFileClass *klass = MAMAN_FILE_CLASS (g_class);

        klass->write_signal_id = 
                g_signal_newv ("write",
                               G_TYPE_FROM_CLASS (g_class),
                               G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
                               NULL /* class closure */,
                               NULL /* accumulator */,
                               NULL /* accu_data */,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE /* return_type */,
                               0     /* n_params */,
                               NULL  /* param_types */);

}

GType maman_file_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (MamanFileClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        maman_file_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (MamanFile),
                        0,      /* n_preallocs */
                        maman_file_instance_init    /* instance_init */
                };
                type = g_type_register_static (G_TYPE_OBJECT,
                                               "MamanFileType",
                                               &info, 0);
        }
        return type;
}

MamanFile *maman_file_new(void)
{
	return g_object_new(maman_file_get_type(), NULL);
}

static DBusHandlerResult maman_file_handle_icd_message(DBusConnection *c,
							DBusMessage *msg,
							gpointer user_data)
{
	guint iap_id;
	DBusHandlerResult ret;

	g_debug("maman_file handle_icd_message()");
	if(!dbus_message_is_signal(msg, "org.gnome.Rhythmbox.Player",
				"elapsedChanged")) {
		ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
		goto exit;
	}

	if(!dbus_message_get_args(msg, NULL,
				DBUS_TYPE_UINT32, &iap_id,
				DBUS_TYPE_INVALID)) {
		ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
		goto exit;
	}
	
	ret = DBUS_HANDLER_RESULT_HANDLED;
exit:
	return ret;
}
		
static DBusConnection *maman_file_get_dbus_connection(MamanFile *file)
{
	static DBusConnection *dbus_connection = NULL;

	static struct DBusObjectPathVTable icd_vtable = {
		.message_function = &maman_file_handle_icd_message
	};

	g_debug("maman_file_get_dbus_connection()");

	if (dbus_connection == NULL){
		dbus_connection = dbus_bus_get(DBUS_BUS_SESSION, NULL);

		if(dbus_connection == NULL){
			g_critical("Can't get session bus");
			goto exit;
		}

		dbus_bus_add_match(dbus_connection,
				"type='signal',"
				"interface='org.gnome.Rhythmbox.Player',",
				NULL);

		g_debug("registering handle_icd_message");
		if(!dbus_connection_register_object_path(dbus_connection,
					"/org/gnome/Rhythmbox/Player",
					&icd_vtable,
					NULL)){
			g_debug("failed to register handler");
			dbus_connection_close(dbus_connection);
			dbus_connection_unref(dbus_connection);
			dbus_connection = NULL;	
		}
	}
exit:
	return file->private->dbus_connection = dbus_connection;
}

void maman_file_write (MamanFile *self, gint buffer, gint size)
{
        /* First write data. */
        /* Then, notify user of data written. */
	DBusConnection *conn = maman_file_get_dbus_connection(self);

	g_debug("maman_file_write()");
        g_signal_emit (self, MAMAN_FILE_GET_CLASS (self)->write_signal_id,
                       0 /* details */, 
                       NULL);
}
